"""Add song filename of every file played to completion to playlist.txt."""

import os
from quodlibet import const
from quodlibet.plugins.events import EventPlugin


class AddToListPlugin(EventPlugin):
    """Create a playlist of played songs."""
    PLUGIN_ID = "Add to list"
    PLUGIN_NAME = _("Add To Playlist")
    PLUGIN_DESC = _("Add filename of any played song to file playlist.txt.")
    PLUGIN_ICON = "gtk-find-and-replace"
    PLUGIN_VERSION = "0.1"

    def player_get_userdir(self):
        """get the application user directory to store files"""
        try:
            return const.USERDIR
        except AttributeError:
            return const.DIR

    def plugin_on_song_ended(self, song, skipped):
        """Triggered when song ends/is skipped"""
        if song is None:
            return
        if not skipped:
            with open(os.path.join(self.player_get_userdir(),
                                   'playlist.txt'), 'a') as playlist:
                title = song('title')
                version = song('version')
                if version:
                    title += ' (%s)' % version
                playlist.write('# %s - %s\n' % (song('artist'), title))
                playlist.write('%s\n' % song('~filename'))
