#!/usr/bin/python
"""Update spam learning."""

import logging
import notmuch
import subprocess
from time import time
import sys

def tag_message(msg, *tags):
    msg.freeze()
    for tag in tags:
        if tag[0] == '+':
            msg.add_tag(tag[1:])
        elif tag[0] == '-':
            msg.remove_tag(tag[1:])
        else:
            msg.add_tag(tag)
    msg.thaw()

def tag_search(db, search, *tags):
    q = notmuch.Query(db, search)
    count = 0
    for msg in q.search_messages():
        count += 1
        tag_message(msg, *tags)
    if count > 0:
        logging.debug(
            'Tagging %d messages with (%s)', count, ' '.join(tags))

logging.basicConfig(level=logging.INFO)

db = notmuch.Database(mode=notmuch.Database.MODE.READ_WRITE)
if '--clean' in sys.argv:
    import shutil
    import os.path
    shutil.rmtree(os.path.expanduser('~/.bogofilter'))
    tag_search(db, 'tag:.bf_spam', '-.bf_spam')
    tag_search(db, 'tag:.bf_ham', '-.bf_ham')


def do_update(search, tag_func, bf_args):
    start_time = time()
    p = subprocess.Popen(['bogofilter', bf_args], stdin=subprocess.PIPE)
    q = notmuch.Query(db, search)
    n = 0
    for msg in q.search_messages():
        p.stdin.write('%s\n' % msg.get_filename())
        tag_func(msg)
        n += 1
    p.stdin.close()
    p.wait()
    return (n, time() - start_time)

logging.info('Registering spam')
n, t = do_update(
    'tag:junk and not tag:.bf_spam',
    lambda msg: msg.add_tag('.bf_spam'), '-sb')
logging.info('Registered %d spam in %1.2f seconds', n, t)

logging.info('Unregistering spam')
n, t = do_update(
    'not tag:junk and tag:.bf_spam',
    lambda msg: msg.remove_tag('.bf_spam'), '-Sb')
logging.info('Unregistered %d spam in %1.2f seconds', n, t)

# Only consider messages that have been read as ham
logging.info('Registering ham')
n, t = do_update(
    'not tag:junk and not tag:unread and not tag:.bf_ham',
    lambda msg: msg.add_tag('.bf_ham'), '-nb')
logging.info('Registered %d ham in %1.2f seconds', n, t)

logging.info('Unregistering ham')
n, t = do_update(
    'tag:junk and tag:.bf_ham',
    lambda msg: msg.remove_tag('.bf_ham'), '-Nb')
logging.info('Unregistered %d ham in %1.2f seconds', n, t)

